﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Dto;
using Model.OrderCriteria;
using Shared;
using Asp.Versioning;

namespace API.Controllers
{
    [Route("api/v{version:apiVersion}/[controller]")]
    [Authorize]
    [ApiVersion("1.0")]
    [ApiController]
    public class InquiriesController : Controller
    {
        private IdataService<UserDTO, LessonDTO, InquiryDTO, ParagraphDTO, SuccessDTO> _dataService;

        private readonly ILogger<InquiriesController> _logger;
        public InquiriesController(IdataService<UserDTO, LessonDTO, InquiryDTO, ParagraphDTO, SuccessDTO> dataService, ILogger<InquiriesController> logger)
        {
            _dataService = dataService;
            _logger = logger;
        }

        [HttpGet("inquiries/{page}/{number}/{orderCriteria}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 204)]
        public async Task<IActionResult> GetInquiries(int page, int number, InquiryOrderCriteria orderCriteria)
        {
            var inquiries = ( await _dataService.inquiryService.GetItems(page, number, orderCriteria)).ToList();
            if (inquiries.Count() == 0)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée.");
                return StatusCode(204);
            }
            _logger.LogInformation("[INFORMATION] {nb} Enquête(s) trouvée(s)", inquiries.Count);
            return Ok(inquiries);
        }

        [HttpGet("inquiry/{id:int}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 404)]
        public async Task<IActionResult> GetInquiryById(int id)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] L'enquête avec l'id {id} a été trouvé.", id);
                return Ok( await _dataService.inquiryService.GetItems(1, 1, null, InquiryOrderCriteria.ById.ToString().Substring(2),id));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée avec l'id {id}.", id);
                return NotFound();
            }
        }

        [HttpGet("inquiry/{title:alpha}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 404)]
        public async Task<IActionResult> GetInquiryByTitle(string title)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] L'enquête avec le titre {title} a été trouvé.", title);
                return Ok( await _dataService.inquiryService.GetItems(1, 1, null, InquiryOrderCriteria.ByTitle.ToString().Substring(2), title));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée avec le titre {title}.", title);
                return NotFound();
            }
        }

        [HttpDelete("inquiry/{id}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 404)]
        public async Task<IActionResult> DeleteInquiry(int id)
        {
            var success = await _dataService.inquiryService.DeleteItem(id);
            if (success)
            {
                _logger.LogInformation("[INFORMATION] L'enquête avec l'id {id} a été supprimé.", id);
                return Ok(_dataService.inquiryService.DeleteItem(id));
            }
            else
            {
                _logger.LogError("[ERREUR] Aucune enquête trouvée avec l'id {id}.", id);
                return NotFound();
            }

        }

        [HttpPost]
        [ProducesResponseType(typeof(InquiryDTO), 201)]
        [ProducesResponseType(typeof(string), 400)]
        public async Task<IActionResult> CreateInquiry([FromBody] InquiryDTO dto)
        {
            if (dto.Title == null || dto.Description == null)
            {
                return BadRequest();
            }
            _logger.LogInformation("[INFORMATION] Une enquête a été créé : title - {title}, description - {description}, isUser - {isUser}", dto.Title, dto.Description, dto.IsUser);
            return Created(nameof(GetInquiries), await _dataService.inquiryService.AddItem(dto));
        }

        [HttpPut("inquiry/{id}")]
        [ProducesResponseType(typeof(InquiryDTO), 200)]
        [ProducesResponseType(typeof(string), 400)]
        [ProducesResponseType(typeof(string), 404)]
        public async Task<IActionResult> UpdateInquiry(int id, [FromBody] InquiryDTO inquiryDTO)
        {
            if (id != inquiryDTO.Id)
            {
                _logger.LogError("[ERREUR] Problème ID - La mise à jour de l'enquête avec l'id {id} a échouée.", id);
                return BadRequest();
            }
            if (!ModelState.IsValid)
            {
                _logger.LogError("[ERREUR] Problème controlleur - La mise à jour de l'enquête avec l'id {id} a échouée.", id);
                return BadRequest();
            }
            if (inquiryDTO != null)
            {
                _logger.LogInformation("[INFORMATION] La mise à jour de l'enquête avec l'id {id} a été effectuée", id);
                return Ok( await _dataService.inquiryService.UpdateItem<InquiryDTO>(id, inquiryDTO));
            }
            _logger.LogError("[ERREUR] Aucune enquête trouvée avec l'id {id}.", id);
            return NotFound();
        }
    }
}